# frozen_string_literal: true

module Hermes
  # Entity serializer
  class EntitySerializer < Hermes::ApplicationSerializer
    attributes %i[
      als_id
      title
      siret
      siren
      is_root
      root_id
      parent_id
      id_rex
      id_cristal
      automated_request_validations
      delegated_validations
      priorities_enabled
      in_peec
      achieved_goal
      spent_on_completed_requests
      spent_on_ongoing_requests
      spent_on_other
      annual_stock
      total_spent
      available_points
      applicant_ids
      housing_offer_ids
      lessor_id
      group_id
      supervisor_group_id
      manager_group_id
      auto_validation
      current_user_right
      manual_mode
    ]

    type 'entities'
def manual_mode
  object&.manual_mode if object.respond_to? :manual_mode
end
    def applicant_ids
      object.applicant_ids&.map(&:to_s)
    end

    def housing_offer_ids
      object.housing_offer_ids&.map(&:to_s)
    end

    def lessor_id
      object.lessor_id&.to_s
    end

    def group_id
      object.group_id&.to_s
    end

    def supervisor_group_id
      object.supervisor_group_id
    end

    def manager_group_id
      object.manager_group_id
    end

    def current_user_right
      right = nil
      current_user.group_ids.each do |id|
        right = 'reader' if id == object.group_id
        right = 'user' if id == object.supervisor_group_id
        right = 'administrator' if id == object.manager_group_id
      end
      right
    end
  end
end
